@@ -1,6 +1,6 @@
 from .layer import wrap, layer_map as key
 from time import time
-from .util import np
+from .util import np, clear_buf
 
 class Net:
     def __init__(self):
@@ -45,7 +45,7 @@ class Net:
                     p = [rst[i] for i in out]
                 else: p = [rst[out]]
                 xs = x if isinstance(x, list) else [x]
-                for k in xs: # release wasted obj
+                for k in set(xs): # release wasted obj
                     if self.life[k]<=i: del rst[k]
                 obj = dic[l]
                 start = time()
@@ -63,13 +63,19 @@ class Net:
                 if not obj.name in self.timer:
                     self.timer[obj.name] = 0
                 self.timer[obj.name] += cost
+        clear_buf()
         return rst[y][0] if len(rst[y])==1 else rst[y]
 
+    def timeit(self, status='start'):
+        if status == 'start': self.timer = {}
+        if status == 'end':
+            for i in self.timer: print(i, self.timer[i])
+
     def run(self, output=None, input={}):
-        return self.forward(input) # compatible with onnxruntime
+        rst = self.forward(input) # compatible with onnxruntime
+        return rst if isinstance(rst, tuple) else (rst,)
 
     def load_weights(self, data):
-        import numpy as cpu
         s, data = 0, data.view(dtype=np.uint8)
         for i in range(len(self.weights)):
             buf = self.weights[i].view(dtype=np.uint8)
